'use strict';
const {src, dest, watch, series, parallel} = require('gulp');
const sourcemaps = require('gulp-sourcemaps');
const sass = require('gulp-dart-sass');
const sassLint = require('gulp-sass-lint');
const uglify = require('gulp-uglify');
const postcss = require('gulp-postcss');
const autoprefixer = require('autoprefixer');
const cssnano = require('cssnano');
const cache = require('gulp-cache');
const imagemin = require('gulp-imagemin');
const imageminPngquant = require('imagemin-pngquant');
const imageminZopfli = require('imagemin-zopfli');
const imageminMozjpeg = require('imagemin-mozjpeg');
const imageminGiflossy = require('imagemin-giflossy');
const eslint = require('gulp-eslint');
const browserSync = require('browser-sync').create();
const mode = require('gulp-mode')();
const exec = require('child_process').exec;

const configPath = 'dev/tools/soul/config/';
const projectPath = '../../../';
const theme = require(projectPath + 'dev/tools/soul/config/themes.json');

// Sass task
function scssTask() {
    return src(projectPath + theme.base.src + theme.base.stylesDir + '/**/*.s+(a|c)ss')
        .pipe(sourcemaps.init())
        .pipe(sass(
            {
                includePaths: ['./node_modules/susy/sass', './node_modules/breakpoint-sass/stylesheets/breakpoint'],
            }
        ).on('error', sass.logError))
        .pipe(postcss([autoprefixer(), cssnano()]))
        .pipe(sourcemaps.write('.'))
        .pipe(dest(projectPath + theme.base.dest + 'css'))
        .pipe(browserSync.stream())
}

// Sass Lint
function sasslint() {
    return src(projectPath + theme.base.src + theme.base.stylesDir + '/**/*.s+(a|c)ss', '!node_modules/**')
        .pipe(sassLint())
        .pipe(sassLint.format())
        .pipe(sassLint.failOnError());
}

// JS task
function jsTask() {
    return src([projectPath + theme.base.src + '**/*.js', '!node_modules/**'])
        .pipe(uglify())
}

// JS Lint
function jsLint() {
    return src([projectPath + theme.base.src + '**/*.js', '!node_modules/**'])
        .pipe(eslint({
            configFile: projectPath + configPath + '.eslintrc',
            fix: true
        }))
        .pipe(eslint.format())
        .pipe(eslint.failAfterError());
}

// Optimise Images
function images() {
    return src([projectPath + theme.base.src + '**/*.{gif,png,jpg,svg}', '!node_modules/**'])
        .pipe(cache(imagemin([
            imageminPngquant({
                speed: 1,
                quality: [0.65, 0.80]
            }),
            imageminZopfli({
                more: true,
                iterations: 50
            }),
            imagemin.gifsicle({
                interlaced: true,
                optimizationLevel: 3
            }),
            imageminGiflossy({
                optimizationLevel: 3,
                optimize: 3,
                lossy: 2
            }),
            imagemin.svgo({
                plugins: [{
                    removeViewBox: false,
                }]
            }),
            imagemin.mozjpeg({
                progressive: true
            }),
            imageminMozjpeg({
                quality: 75
            })
        ])))
}

// Setup task
function setupTask() {
    return src(['config/*'])
        .pipe(dest(projectPath + configPath + 'config'));
}

// BrowserSync
function reload(done) {
    browserSync.reload();
    done();
}

// Magepack Generate
function magepackGenerate(cb) {
    exec('cd ../../../ && magepack generate --cms-url="' + theme.base.magepackCMS + '" --category-url="' + theme.base.magepackCategory + '" --product-url="' + theme.base.magepackProduct + '"', function (err, stdout, stderr) {
        console.log(stdout);
        console.log(stderr);
        cb(err);
    });
}

// Magepack Bundle
function magepackBundle(cb) {
    exec('cd ../../../ && magepack bundle', function (err, stdout, stderr) {
        console.log(stdout);
        console.log(stderr);
        cb(err);
    });
}

// Magepack ConfigSet
function magepackConfigSet(cb) {
    exec('cd ../../../ && php73 bin/magento config:set dev/js/enable_magepack_js_bundling 1', function (err, stdout, stderr) {
        console.log(stdout);
        console.log(stderr);
        cb(err);
    });
}

// Watch task: watch SCSS, images and JS files for changes
function watchTask() {
    browserSync.init({
        proxy: theme.base.proxy
    });
    watch([projectPath + theme.base.src + theme.base.stylesDir + '/**/*.s+(a|c)ss', projectPath + theme.base.src + '**/*.js', projectPath + theme.base.src + '**/*.{gif,png,jpg,svg}', '!node_modules/**'],
        {interval: 1000, usePolling: true},
        series(
            parallel(scssTask, sasslint, jsTask, jsLint),
        )
    );
}

// Export the default Gulp task so it can be run
// Runs the scss, scss lint, images and js tasks simultaneously
exports.default = series(
    parallel(scssTask, sasslint, jsTask, jsLint, images), reload,
    watchTask
);

exports.build = series(
    parallel(scssTask, sasslint, jsTask, jsLint)
);

exports.magepackGenerate = series(magepackGenerate, magepackBundle, magepackConfigSet);

// Run this task to create configs and linting rules
exports.setup = series(setupTask);
